from bgeditor.dao.ComponentVer2 import  Component
from moviepy.editor import *

import numpy as np
from bgeditor.common.utils import cache_file, download_file, upload_file, upload_static_file, upload_file_resource
import uuid, textwrap
import math

